"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ImportOnDemand = void 0;
/* eslint-disable no-unused-vars */
const path_1 = __importDefault(require("path"));
const Visitor_1 = require("@swc/core/Visitor");
function toArray(data) {
    return Array.isArray(data) ? data : [data];
}
function transCamel(str, symbol) {
    // eslint-disable-next-line no-param-reassign
    str = str[0].toLowerCase() + str.substring(1);
    return str.replace(/([A-Z])/g, $1 => `${symbol}${$1.toLowerCase()}`);
}
function getTransform(config) {
    if (typeof config.transform === 'string' &&
        config.transform.endsWith('.js') &&
        !config.transform.includes('[')) {
        const absolutePath = path_1.default.resolve(process.cwd(), config.transform);
        return require(absolutePath);
    }
    return config.transform;
}
function runTransform(config, transform, ctx) {
    return toArray(typeof transform === 'function' ? transform(ctx) : transform)
        .map(pattern => {
        if (!pattern) {
            // eslint-disable-next-line no-undefined
            return undefined;
        }
        return pattern.replace(/\[(.+?)(?::(.+?))?\]/g, (_, key, symbol) => {
            if (!(key in ctx)) {
                throw new Error(`[swc-plugin-import-on-demand] Unexpected key \`${key}\` in \`${pattern}\``);
            }
            return symbol ? transCamel(ctx[key], symbol) : ctx[key];
        });
    })
        .filter((x) => !!x);
}
const wrapStr = (s, wrapChar) => `${wrapChar}${s}${wrapChar}`;
const isDoubleQuote = (s) => s && s[0] === '"';
class ImportOnDemand extends Visitor_1.Visitor {
    constructor(userConfigMap) {
        super();
        this.configMap = Object.keys(userConfigMap).reduce((res, key) => {
            const userConfig = userConfigMap[key];
            res[key] =
                typeof userConfig === 'string' ||
                    typeof userConfig === 'function' ||
                    // eslint-disable-next-line prettier/prettier
                    Array.isArray(userConfig) ? { transform: userConfig } : userConfig;
            return res;
        }, {});
    }
    visitModuleItems(items) {
        var _a;
        const newItems = [];
        for (const item of items) {
            if (item.type !== 'ImportDeclaration' || item.typeOnly) {
                newItems.push(item);
                continue;
            }
            const { source, specifiers } = item;
            const config = this.configMap[source.value];
            if (!config) {
                newItems.push(item);
                continue;
            }
            const transform = getTransform(config);
            for (const specifier of specifiers) {
                if (specifier.type !== 'ImportSpecifier') {
                    newItems.push(item);
                    continue;
                }
                const name = ((_a = specifier.imported) === null || _a === void 0 ? void 0 : _a.value) || specifier.local.value;
                runTransform(config, transform, {
                    source: source.value,
                    name,
                }).map((transformed, index) => {
                    const quoteMark = isDoubleQuote(item.source.raw) ? '"' : "'";
                    const newSource = {
                        ...item.source,
                        value: transformed,
                        raw: wrapStr(transformed, quoteMark),
                    };
                    if (index === 0) {
                        const newSpecifier = {
                            ...specifier,
                            type: 'ImportDefaultSpecifier',
                        };
                        newItems.push({
                            ...item,
                            source: newSource,
                            specifiers: [newSpecifier],
                        });
                    }
                    else {
                        newItems.push({
                            ...item,
                            source: newSource,
                            specifiers: [],
                        });
                    }
                });
            }
        }
        return newItems;
    }
}
exports.ImportOnDemand = ImportOnDemand;
exports.default = ImportOnDemand;
